@@ -56,7 +56,7 @@ class DibImageFile(object):
     a unique ID, but can be available in multiple formats (with different
     extensions).
     '''
-    def __init__(self, image_id, extension=None):
+    def __init__(self, image_id, extension):
         self.image_id = image_id
         self.extension = extension
         self.md5 = None
@@ -69,34 +69,18 @@ class DibImageFile(object):
         if self.extension:
             self.extension = self.extension.lstrip('.')
 
-    @staticmethod
-    def from_path(path):
-        if isinstance(path, str):
-            path = Path(path)
-        return DibImageFile(path.stem, path.suffix)
-
     @staticmethod
     def from_image_id(images_dir, image_id):
         images = []
         for image_file in Path(images_dir).iterdir():
             if image_file.is_file():
-                image = DibImageFile.from_path(image_file)
+                image = DibImageFile(image_file.stem, image_file.suffix)
                 if image.image_id == image_id:
                     images.append(image)
         return images
 
-    @staticmethod
-    def from_images_dir(images_dir):
-        return [DibImageFile.from_path(x) for x in Path(images_dir).iterdir()]
-
-    def to_path(self, images_dir, with_extension=True):
-        my_path = Path(images_dir) / self.image_id
-        if with_extension:
-            if self.extension is None:
-                raise exceptions.BuilderError(
-                    'Cannot specify image extension of None'
-                )
-            my_path = my_path.with_suffix('.' + self.extension)
+    def to_path(self, images_dir):
+        my_path = Path(images_dir) / f'{self.image_id}.{self.extension}'
 
         # Path.with_suffix() will replace an existing suffix, so we create
         # new Path objects from strings for the checksum files.
@@ -270,7 +254,7 @@ class CleanupWorker(BaseWorker):
         manifest_dir = None
 
         for f in files:
-            filename = f.to_path(images_dir, True)
+            filename = f.to_path(images_dir)
             if not manifest_dir:
                 path, ext = filename.rsplit('.', 1)
                 manifest_dir = path + ".d"
@@ -799,13 +783,15 @@ class BuildWorker(BaseWorker):
         :raises: BuilderError if we failed to execute the build command.
         '''
         base = "-".join([diskimage.name, build_id])
-        image_file = DibImageFile(base)
-        filename = image_file.to_path(self._config.images_dir, False)
+        image_filename = Path(self._config.images_dir) / base
 
         env = os.environ.copy()
         env['DIB_RELEASE'] = diskimage.release
         env['DIB_IMAGE_NAME'] = diskimage.name
-        env['DIB_IMAGE_FILENAME'] = filename
+        # This is just the basename.  DIB may create multiple images
+        # here based on img_types; each will produce a final output
+        # file image_filename.<type>
+        env['DIB_IMAGE_FILENAME'] = image_filename
 
         # Note we use a reference to the nodepool config here so
         # that whenever the config is updated we get up to date
@@ -830,7 +816,7 @@ class BuildWorker(BaseWorker):
 
         cmd = ('%s -x -t %s --checksum --no-tmpfs %s -o %s %s' %
                (dib_cmd, img_types, qemu_img_options,
-                filename, img_elements))
+                image_filename, img_elements))
 
         self._pruneBuildLogs(diskimage.name)
         log_fn = self._getBuildLog(diskimage.name, build_id)
@@ -967,9 +953,10 @@ class BuildWorker(BaseWorker):
                     # purposes of watching if we've added too much stuff
                     # into the image.  Note that st_blocks is defined as
                     # 512-byte blocks by stat(2)
-                    size = os.stat("%s.%s" % (filename, ext)).st_blocks * 512
+                    size = os.stat("%s.%s" %
+                                   (image_filename, ext)).st_blocks * 512
                     self.log.debug("%s created image %s.%s (size: %d) " %
-                                   (diskimage.name, filename, ext, size))
+                                   (diskimage.name, image_filename, ext, size))
                     pipeline.gauge(key, size)
 
         if self._statsd:
@@ -1078,10 +1065,12 @@ class UploadWorker(BaseWorker):
         self.log.debug("Found image file of type %s for image id: %s" %
                        (image.extension, image.image_id))
 
-        filename = image.to_path(self._config.images_dir, with_extension=True)
+        filename = image.to_path(self._config.images_dir)
 
         ext_image_name = provider.image_name_format.format(
-            image_name=image_name, timestamp=str(timestamp)
+            image_name=image_name,
+            upload_id=upload_id,
+            timestamp=str(timestamp)
         )
 
         self.log.info("Uploading DIB image build %s from %s to %s" %
@@ -1100,6 +1089,7 @@ class UploadWorker(BaseWorker):
 
         try:
             external_id = manager.uploadImage(
+                provider_image,
                 ext_image_name, filename,
                 image_type=image.extension,
                 meta=meta,
@@ -1407,7 +1397,9 @@ class NodePoolBuilder(object):
                 list(self._config.zookeeper_servers.values()),
                 tls_cert=self._config.zookeeper_tls_cert,
                 tls_key=self._config.zookeeper_tls_key,
-                tls_ca=self._config.zookeeper_tls_ca)
+                tls_ca=self._config.zookeeper_tls_ca,
+                timeout=self._config.zookeeper_timeout,
+            )
 
             self.log.debug('Starting listener for build jobs')
 
