@@ -131,7 +131,7 @@ class Driver(object, metaclass=abc.ABCMeta):
 
 class ProviderNotifications(object):
     """
-    Notification interface for :class:`.Provider` objects.
+    Notification interface for :py:class:`.Provider` objects.
 
     This groups all notification messages bound for the Provider. The
     Provider class inherits from this by default. A Provider overrides the
@@ -326,7 +326,7 @@ class LabelRecorder(object):
 
 class NodeRequestHandlerNotifications(object):
     """
-    Notification interface for :class:`.NodeRequestHandler` objects.
+    Notification interface for :py:class:`.NodeRequestHandler` objects.
 
     This groups all notification messages bound for the NodeRequestHandler.
     The NodeRequestHandler class inherits from this by default. A request
@@ -466,6 +466,7 @@ class NodeRequestHandler(NodeRequestHandlerNotifications,
                         got_a_node = True
                         node.allocated_to = self.request.id
                         node.tenant_name = self.request.tenant_name
+                        node.requestor = self.request.requestor
                         self.zk.storeNode(node)
                         self.nodeset.append(node)
                         self._satisfied_types.add(ntype, node.id)
@@ -514,6 +515,7 @@ class NodeRequestHandler(NodeRequestHandlerNotifications,
                 node.launcher = self.launcher_id
                 node.allocated_to = self.request.id
                 node.tenant_name = self.request.tenant_name
+                node.requestor = self.request.requestor
 
                 # This sets static data defined in the config file in the
                 # ZooKeeper Node object.
