@@ -12,10 +12,12 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import os
-import math
-import logging
 import json
+import logging
+import math
+import os
+import random
+import string
 
 import cachetools.func
 
@@ -41,11 +43,24 @@ def quota_info_from_sku(sku):
         instances=1)
 
 
+def generate_password():
+    while True:
+        chars = random.choices(string.ascii_lowercase +
+                               string.ascii_uppercase +
+                               string.digits,
+                               k=64)
+        if ((set(string.ascii_lowercase) & set(chars)) and
+            (set(string.ascii_uppercase) & set(chars)) and
+            (set(string.digits) & set(chars))):
+            return(''.join(chars))
+
+
 class AzureInstance(statemachine.Instance):
     def __init__(self, vm, nic=None, public_ipv4=None,
                  public_ipv6=None, sku=None):
+        super().__init__()
         self.external_id = vm['name']
-        self.metadata = vm['tags'] or {}
+        self.metadata = vm.get('tags', {})
         self.private_ipv4 = None
         self.private_ipv6 = None
         self.public_ipv4 = None
@@ -160,6 +175,7 @@ class AzureCreateStateMachine(statemachine.StateMachine):
         self.retries = retries
         self.attempts = 0
         self.image_external_id = image_external_id
+        self.image_reference = None
         self.metadata = metadata
         self.tags = label.tags.copy() or {}
         self.tags.update(metadata)
@@ -189,6 +205,12 @@ class AzureCreateStateMachine(statemachine.StateMachine):
     def advance(self):
         if self.state == self.START:
             self.external_id = self.hostname
+
+            # Find an appropriate image if filters were provided
+            if self.label.cloud_image and self.label.cloud_image.image_filter:
+                self.image_reference = self.adapter._getImageFromFilter(
+                    self.label.cloud_image.image_filter)
+
             if self.label.pool.public_ipv4:
                 self.public_ipv4 = self.adapter._createPublicIPAddress(
                     self.tags, self.hostname, self.ip_sku, 'IPv4',
@@ -220,8 +242,9 @@ class AzureCreateStateMachine(statemachine.StateMachine):
             self.nic = self.adapter._refresh(self.nic)
             if self.adapter._succeeded(self.nic):
                 self.vm = self.adapter._createVirtualMachine(
-                    self.label, self.image_external_id, self.tags,
-                    self.hostname, self.nic)
+                    self.label, self.image_external_id,
+                    self.image_reference, self.tags, self.hostname,
+                    self.nic)
                 self.state = self.VM_CREATING
             else:
                 return
@@ -306,11 +329,11 @@ class AzureAdapter(statemachine.Adapter):
         self._getSKUs()
 
     def getCreateStateMachine(self, hostname, label,
-                              image_external_id, metadata, retries):
+                              image_external_id, metadata, retries, log):
         return AzureCreateStateMachine(self, hostname, label,
                                        image_external_id, metadata, retries)
 
-    def getDeleteStateMachine(self, external_id):
+    def getDeleteStateMachine(self, external_id, log):
         return AzureDeleteStateMachine(self, external_id)
 
     def listResources(self):
@@ -364,8 +387,9 @@ class AzureAdapter(statemachine.Adapter):
                              self.provider.location))
         return quota_info_from_sku(sku)
 
-    def uploadImage(self, image_name, filename, image_format,
-                    metadata, md5, sha256):
+    def uploadImage(self, provider_image, image_name, filename,
+                    image_format, metadata, md5, sha256):
+        self.log.debug(f"Uploading image {image_name}")
         file_sz = os.path.getsize(filename)
         disk_info = {
             "location": self.provider.location,
@@ -449,13 +473,16 @@ class AzureAdapter(statemachine.Adapter):
         if r['status'] != 'Succeeded':
             raise Exception("Unable to delete disk for image upload")
 
+        self.log.info(f"Uploaded image {image_name}")
         return image_name
 
     def deleteImage(self, external_id):
+        self.log.debug(f"Deleting image {external_id}")
         with self.rate_limiter:
             r = self.azul.images.delete(self.resource_group, external_id)
         r = self.azul.wait_for_async_operation(r)
 
+        self.log.info(f"Deleted image {external_id}")
         if r['status'] != 'Succeeded':
             raise Exception("Unable to delete image")
 
@@ -542,10 +569,12 @@ class AzureAdapter(statemachine.Adapter):
                 'publicIpAllocationMethod': allocation_method,
             },
         }
+        name = "%s-pip-%s" % (hostname, version)
         with self.rate_limiter:
+            self.log.debug(f"Creating external IP address {name}")
             return self.azul.public_ip_addresses.create(
                 self.resource_group,
-                "%s-pip-%s" % (hostname, version),
+                name,
                 v4_params_create,
             )
 
@@ -556,6 +585,7 @@ class AzureAdapter(statemachine.Adapter):
         else:
             return None
         with self.rate_limiter:
+            self.log.debug(f"Deleting external IP address {name}")
             self.azul.public_ip_addresses.delete(self.resource_group, name)
         return pip
 
@@ -601,10 +631,12 @@ class AzureAdapter(statemachine.Adapter):
                 'ipConfigurations': ip_configs
             }
         }
+        name = "%s-nic" % hostname
         with self.rate_limiter:
+            self.log.debug(f"Creating NIC {name}")
             return self.azul.network_interfaces.create(
                 self.resource_group,
-                "%s-nic" % hostname,
+                name,
                 nic_data
             )
 
@@ -615,6 +647,7 @@ class AzureAdapter(statemachine.Adapter):
         else:
             return None
         with self.rate_limiter:
+            self.log.debug(f"Deleting NIC {name}")
             self.azul.network_interfaces.delete(self.resource_group, name)
         return nic
 
@@ -623,17 +656,27 @@ class AzureAdapter(statemachine.Adapter):
         with self.rate_limiter:
             return self.azul.virtual_machines.list(self.resource_group)
 
-    def _createVirtualMachine(self, label, image_external_id, tags,
-                              hostname, nic):
+    def _createVirtualMachine(self, label, image_external_id,
+                              image_reference, tags, hostname, nic):
         if image_external_id:
+            # This is a diskimage
             image = label.diskimage
             remote_image = self._getImage(image_external_id)
             image_reference = {'id': remote_image['id']}
-        else:
+        elif image_reference:
+            # This is a cloud image with aser supplied image-filter;
+            # we already found the reference.
             image = label.cloud_image
-            image_reference = label.cloud_image.image_reference
+        else:
+            # This is a cloud image with a user-supplied reference or
+            # id.
+            image = label.cloud_image
+            if label.cloud_image.image_reference:
+                image_reference = label.cloud_image.image_reference
+            else:
+                image_reference = {'id': label.cloud_image.image_id}
         os_profile = {'computerName': hostname}
-        if image.username and image.key:
+        if image.key:
             linux_config = {
                 'ssh': {
                     'publicKeys': [{
@@ -644,40 +687,53 @@ class AzureAdapter(statemachine.Adapter):
                 },
                 "disablePasswordAuthentication": True,
             }
-            os_profile['adminUsername'] = image.username
             os_profile['linuxConfiguration'] = linux_config
+        if image.username:
+            os_profile['adminUsername'] = image.username
+        if image.password:
+            os_profile['adminPassword'] = image.password
+        elif image.generate_password:
+            os_profile['adminPassword'] = generate_password()
+        if label.custom_data:
+            os_profile['customData'] = label.custom_data
 
+        spec = {
+            'location': self.provider.location,
+            'tags': tags,
+            'properties': {
+                'osProfile': os_profile,
+                'hardwareProfile': {
+                    'vmSize': label.hardware_profile["vm-size"]
+                },
+                'storageProfile': {
+                    'imageReference': image_reference,
+                },
+                'networkProfile': {
+                    'networkInterfaces': [{
+                        'id': nic['id'],
+                        'properties': {
+                            'primary': True,
+                        }
+                    }]
+                },
+            },
+        }
+        if label.user_data:
+            spec['properties']['userData'] = label.user_data
         with self.rate_limiter:
+            self.log.debug(f"Creating VM {hostname}")
             return self.azul.virtual_machines.create(
-                self.resource_group, hostname, {
-                    'location': self.provider.location,
-                    'tags': tags,
-                    'properties': {
-                        'osProfile': os_profile,
-                        'hardwareProfile': {
-                            'vmSize': label.hardware_profile["vm-size"]
-                        },
-                        'storageProfile': {
-                            'imageReference': image_reference,
-                        },
-                        'networkProfile': {
-                            'networkInterfaces': [{
-                                'id': nic['id'],
-                                'properties': {
-                                    'primary': True,
-                                }
-                            }]
-                        },
-                    },
-                })
+                self.resource_group, hostname, spec)
 
     def _deleteVirtualMachine(self, name):
         for vm in self._listVirtualMachines():
             if vm['name'] == name:
                 break
         else:
+            self.log.warning(f"VM not found when deleting {name}")
             return None
         with self.rate_limiter:
+            self.log.debug(f"Deleting VM {name}")
             self.azul.virtual_machines.delete(self.resource_group, name)
         return vm
 
@@ -701,3 +757,24 @@ class AzureAdapter(statemachine.Adapter):
     def _listImages(self):
         with self.rate_limiter:
             return self.azul.images.list(self.resource_group)
+
+    def _getImageFromFilter(self, image_filter):
+        images = self._listImages()
+        images = [i for i in images
+                  if i['properties']['provisioningState'] == 'Succeeded']
+        if 'name' in image_filter:
+            images = [i for i in images
+                      if i['name'] == image_filter['name']]
+        if 'location' in image_filter:
+            images = [i for i in images
+                      if i['location'] == image_filter['location']]
+        if 'tags' in image_filter:
+            for k, v in image_filter['tags'].items():
+                images = [i for i in images if i['tags'].get(k) == v]
+        images = sorted(images, key=lambda i: i['name'])
+        if not images:
+            raise Exception("Unable to find image matching filter: %s",
+                            image_filter)
+        image = images[-1]
+        self.log.debug("Found image matching filter: %s", image)
+        return {'id': image['id']}
