@@ -16,6 +16,7 @@
 # limitations under the License.
 
 import voluptuous as v
+import base64
 import os
 
 from nodepool.driver import ConfigPool
@@ -31,10 +32,15 @@ class AzureProviderCloudImage(ConfigValue):
         }
         self.name = image['name']
         self.username = image['username']
+        self.password = image.get('password')
+        self.generate_password = image.get('generate-password', False)
         # TODO(corvus): remove zuul_public_key
         self.key = image.get('key', zuul_public_key)
-        self.image_reference = image['image-reference']
-        self.python_path = image.get('python-path')
+        self.image_reference = image.get('image-reference')
+        self.image_filter = image.get('image-filter')
+        self.image_id = image.get('image-id')
+        self.python_path = image.get('python-path', 'auto')
+        self.shell_type = image.get('shell-type')
         self.connection_type = image.get('connection-type', 'ssh')
         self.connection_port = image.get(
             'connection-port',
@@ -43,7 +49,8 @@ class AzureProviderCloudImage(ConfigValue):
     @property
     def external_name(self):
         '''Human readable version of external.'''
-        return self.image_id or self.name
+        return (self.image_id or self.image_reference or
+                self.image_filter or self.name)
 
     @staticmethod
     def getSchema():
@@ -54,17 +61,34 @@ class AzureProviderCloudImage(ConfigValue):
             v.Required('offer'): str,
         }
 
-        return {
+        azure_image_filter = {
+            'location': str,
+            'name': str,
+            'tags': dict,
+        }
+
+        return v.All({
             v.Required('name'): str,
             v.Required('username'): str,
+            'password': str,
+            'generate-password': bool,
             # TODO(corvus): make required when zuul_public_key removed
             'key': str,
-            v.Required('image-reference'): azure_image_reference,
+            v.Exclusive('image-reference', 'spec'): azure_image_reference,
+            v.Exclusive('image-id', 'spec'): str,
+            v.Exclusive('image-filter', 'spec'): azure_image_filter,
             'connection-type': str,
             'connection-port': int,
             'python-path': str,
-            # TODO(corvus): shell-type
-        }
+            'shell-type': str,
+        }, {
+            v.Required(
+                v.Any('image-reference', 'image-id', 'image-filter'),
+                msg=('Provide either "image-reference", '
+                     '"image-filter", or "image-id" keys')
+            ): object,
+            object: object,
+        })
 
 
 class AzureProviderDiskImage(ConfigValue):
@@ -76,8 +100,11 @@ class AzureProviderDiskImage(ConfigValue):
         self.name = image['name']
         diskimage.image_types.add('vhd')
         self.pause = bool(image.get('pause', False))
-        self.python_path = image.get('python-path')
+        self.python_path = image.get('python-path', 'auto')
+        self.shell_type = image.get('shell-type')
         self.username = image.get('username')
+        self.password = image.get('password')
+        self.generate_password = image.get('generate-password', False)
         self.key = image.get('key')
         self.connection_type = image.get('connection-type', 'ssh')
         self.connection_port = image.get(
@@ -100,7 +127,7 @@ class AzureProviderDiskImage(ConfigValue):
             'connection-type': str,
             'connection-port': int,
             'python-path': str,
-            # TODO(corvus): shell-type
+            'shell-type': str,
         }
 
 
@@ -141,6 +168,13 @@ class AzureLabel(ConfigValue):
 
         self.hardware_profile = label['hardware-profile']
         self.tags = label.get('tags', {})
+        self.user_data = self._encodeData(label.get('user-data', None))
+        self.custom_data = self._encodeData(label.get('custom-data', None))
+
+    def _encodeData(self, s):
+        if not s:
+            return None
+        return base64.b64encode(s.encode('utf8')).decode('utf8')
 
     @staticmethod
     def getSchema():
@@ -154,6 +188,8 @@ class AzureLabel(ConfigValue):
             'diskimage': str,
             v.Required('hardware-profile'): azure_hardware_profile,
             'tags': dict,
+            'user-data': str,
+            'custom-data': str,
         }
 
 
@@ -166,6 +202,7 @@ class AzurePool(ConfigPool):
         self.load(pool_config)
 
     def load(self, pool_config):
+        super().load(pool_config)
         self.name = pool_config['name']
         self.max_servers = pool_config['max-servers']
         self.public_ipv4 = pool_config.get('public-ipv4',
@@ -181,7 +218,7 @@ class AzurePool(ConfigPool):
         self.use_internal_ip = pool_config.get(
             'use-internal-ip', self.provider.use_internal_ip)
         self.host_key_checking = pool_config.get(
-            'host-key-checking', self.provider.use_internal_ip)
+            'host-key-checking', self.provider.host_key_checking)
 
     @staticmethod
     def getSchema():
@@ -296,9 +333,15 @@ class AzureProviderConfig(ProviderConfig):
             'public-ipv6': bool,
             'use-internal-ip': bool,
             'host-key-checking': bool,
+            'post-upload-hook': str,
+            'rate': v.Coerce(float),
+            'boot-timeout': int,
+            'launch-timeout': int,
+            'launch-retries': int,
             # TODO: remove
             'zuul-public-key': str,
         })
+
         return v.Schema(provider)
 
     def getSupportedLabels(self, pool_name=None):
