@@ -135,14 +135,18 @@ class OpenStackProvider(Provider, QuotaSupport):
                     # It has not leaked.
                     continue
 
-            # In earlier versions of nova, flavor is an id. In later versions
-            # it returns the information we're looking for. If we get the
-            # information, we do not have to attempt to look up the ram or
-            # vcpus.
-            if hasattr(server.flavor, 'id'):
-                flavor = flavors.get(server.flavor.id)
-            else:
+            # In earlier versions of nova or the sdk, flavor has just an id.
+            # In later versions it returns the information we're looking for.
+            # If we get the information we want, we do not need to try to
+            # lookup the flavor in our list.
+            if hasattr(server.flavor, 'vcpus'):
                 flavor = server.flavor
+            else:
+                flavor = flavors.get(server.flavor.id)
+                # If we still haven't found the flavor, skip handling this
+                # server instead of failing completely
+                if not flavor:
+                    continue
             used_quota.add(QuotaInformation.construct_from_flavor(flavor))
 
         return used_quota
@@ -374,8 +378,8 @@ class OpenStackProvider(Provider, QuotaSupport):
             return False
         return True
 
-    def uploadImage(self, image_name, filename, image_type=None, meta=None,
-                    md5=None, sha256=None):
+    def uploadImage(self, provider_image, image_name, filename,
+                    image_type=None, meta=None, md5=None, sha256=None):
         # configure glance and upload image.  Note the meta flags
         # are provided as custom glance properties
         # NOTE: we have wait=True set here. This is not how we normally
@@ -508,7 +512,8 @@ class OpenStackProvider(Provider, QuotaSupport):
         '''
         ret = []
         for p in ports:
-            if p.device_owner is None or p.device_owner.startswith("compute:"):
+            if (p.device_owner is None or p.device_owner == '' or
+                    p.device_owner.startswith("compute:")):
                 ret.append(p)
         return ret
 
