@@ -19,6 +19,7 @@ import math
 from nodepool.driver.taskmanager import BaseTaskManagerProvider, Task
 from nodepool.driver import Driver, NodeRequestHandler
 from nodepool.driver.utils import NodeLauncher, QuotaInformation, QuotaSupport
+from nodepool.driver.utils import NodeDeleter
 from nodepool.nodeutils import iterate_timeout, nodescan
 from nodepool import exceptions
 from nodepool import zk
@@ -153,6 +154,7 @@ class SimpleTaskManagerLauncher(NodeLauncher):
         self.node.host_keys = keys
         self.node.username = self.label.cloud_image.username
         self.node.python_path = self.label.cloud_image.python_path
+        self.node.shell_type = self.label.cloud_image.shell_type
         self.zk.storeNode(self.node)
         self.log.info("Instance %s is ready", hostname)
 
@@ -170,7 +172,7 @@ class SimpleTaskManagerHandler(NodeRequestHandler):
     def alive_thread_count(self):
         count = 0
         for t in self._threads:
-            if t.isAlive():
+            if t.is_alive():
                 count += 1
         return count
 
@@ -354,6 +356,11 @@ class SimpleTaskManagerProvider(BaseTaskManagerProvider, QuotaSupport):
 
         return used_quota
 
+    def startNodeCleanup(self, node):
+        t = NodeDeleter(self._zk, self, node)
+        t.start()
+        return t
+
     def cleanupNode(self, external_id):
         instance = self.getInstance(external_id)
         if (not instance) or instance.deleted:
