@@ -32,7 +32,7 @@ from kazoo import exceptions as kze
 import cachetools
 
 
-def keyscan(node_id, interface_ip,
+def keyscan(host_key_checking, node_id, interface_ip,
             connection_type, connection_port,
             timeout):
     """A standalone function for scanning keys to pass to a thread/process
@@ -40,6 +40,8 @@ def keyscan(node_id, interface_ip,
     """
 
     keys = []
+    if not host_key_checking:
+        return keys
     try:
         if (connection_type == 'ssh' or
             connection_type == 'network_cli'):
@@ -67,7 +69,8 @@ class StateMachineNodeLauncher(stats.StatsReporter):
     def __init__(self, handler, node, provider_config):
         super().__init__()
         # Based on utils.NodeLauncher
-        logger = logging.getLogger("nodepool.StateMachineNodeLauncher")
+        logger = logging.getLogger(
+            f"nodepool.StateMachineNodeLauncher.{provider_config.name}")
         request = handler.request
         self.log = get_annotated_logger(logger,
                                         event_id=request.event_id,
@@ -117,17 +120,19 @@ class StateMachineNodeLauncher(stats.StatsReporter):
 
         self.node.username = image.username
         self.node.python_path = image.python_path
+        self.node.shell_type = image.shell_type
         self.node.connection_port = image.connection_port
         self.node.connection_type = image.connection_type
         self.zk.storeNode(self.node)
 
-        hostname = 'nodepool-' + self.node.id
+        # Windows computer names can be no more than 15 chars long.
+        hostname = 'np' + self.node.id
         retries = self.manager.provider.launch_retries
         metadata = {'nodepool_node_id': self.node.id,
                     'nodepool_pool_name': self.handler.pool.name,
                     'nodepool_provider_name': self.manager.provider.name}
         self.state_machine = self.manager.adapter.getCreateStateMachine(
-            hostname, label, image_external_id, metadata, retries)
+            hostname, label, image_external_id, metadata, retries, self.log)
 
     def updateNodeFromInstance(self, instance):
         if instance is None:
@@ -149,6 +154,16 @@ class StateMachineNodeLauncher(stats.StatsReporter):
         node.public_ipv6 = instance.public_ipv6
         node.region = instance.region
         node.az = instance.az
+        node.driver_data = instance.driver_data
+
+        # Optionally, if the node has updated values that we set from
+        # the image attributes earlier, set those.
+        for attr in ('username', 'python_path', 'shell_type',
+                     'connection_port', 'connection_type',
+                     'host_keys'):
+            if hasattr(instance, attr):
+                setattr(node, attr, getattr(instance, attr))
+
         self.zk.storeNode(node)
 
     def runStateMachine(self):
@@ -164,7 +179,8 @@ class StateMachineNodeLauncher(stats.StatsReporter):
             if (self.state_machine.complete and self.keyscan_future
                 and self.keyscan_future.done()):
                 keys = self.keyscan_future.result()
-                node.keys = keys
+                if keys:
+                    node.host_keys = keys
                 self.log.debug(f"Node {node.id} is ready")
                 node.state = zk.READY
                 self.zk.storeNode(node)
@@ -195,6 +211,7 @@ class StateMachineNodeLauncher(stats.StatsReporter):
                                node.interface_ip)
                 future = self.manager.keyscan_worker.submit(
                     keyscan,
+                    self.handler.pool.host_key_checking,
                     node.id, node.interface_ip,
                     node.connection_type, node.connection_port,
                     self.manager.provider.boot_timeout)
@@ -251,7 +268,11 @@ class StateMachineNodeDeleter:
 
     def __init__(self, zk, provider_manager, node):
         # Based on utils.NodeDeleter
-        self.log = logging.getLogger("nodepool.StateMachineNodeDeleter")
+        logger = logging.getLogger(
+            "nodepool.StateMachineNodeDeleter."
+            f"{provider_manager.provider.name}")
+        self.log = get_annotated_logger(logger,
+                                        node_id=node.id)
         self.manager = provider_manager
         self.zk = zk
         # Note: the node is locked
@@ -259,7 +280,7 @@ class StateMachineNodeDeleter:
         # Local additions:
         self.start_time = time.monotonic()
         self.state_machine = self.manager.adapter.getDeleteStateMachine(
-            node.external_id)
+            node.external_id, self.log)
 
     @property
     def complete(self):
@@ -433,12 +454,12 @@ class StateMachineProvider(Provider, QuotaSupport):
 
     """The Provider implementation for the StateMachineManager driver
        framework"""
-    log = logging.getLogger("nodepool.driver.statemachine."
-                            "StateMachineProvider")
     MINIMUM_SLEEP = 1
     MAXIMUM_SLEEP = 10
 
     def __init__(self, adapter, provider):
+        self.log = logging.getLogger(
+            f"nodepool.StateMachineProvider.{provider.name}")
         super().__init__()
         self.provider = provider
         self.adapter = adapter
@@ -484,7 +505,11 @@ class StateMachineProvider(Provider, QuotaSupport):
         while self.running:
             to_remove = []
             loop_start = time.monotonic()
-            for sm in self.deleters + self.launchers:
+            state_machines = self.deleters + self.launchers
+            if state_machines:
+                self.log.debug("Running %s state machines",
+                               len(state_machines))
+            for sm in state_machines:
                 try:
                     sm.runStateMachine()
                     if sm.complete:
@@ -498,6 +523,9 @@ class StateMachineProvider(Provider, QuotaSupport):
                 if sm in self.launchers:
                     self.launchers.remove(sm)
             loop_end = time.monotonic()
+            if state_machines:
+                self.log.debug("Ran %s state machines in %s seconds",
+                               len(state_machines), loop_end - loop_start)
             if self.launchers or self.deleters:
                 time.sleep(max(0, self.MAXIMUM_SLEEP -
                                (loop_end - loop_start)))
@@ -626,11 +654,12 @@ class StateMachineProvider(Provider, QuotaSupport):
 
     # Image handling
 
-    def uploadImage(self, image_name, filename, image_type=None, meta=None,
-                    md5=None, sha256=None):
+    def uploadImage(self, provider_image, image_name, filename,
+                    image_type=None, meta=None, md5=None, sha256=None):
         meta = meta.copy()
         meta['nodepool_provider_name'] = self.provider.name
-        return self.adapter.uploadImage(image_name, filename,
+        return self.adapter.uploadImage(provider_image, image_name,
+                                        filename,
                                         image_format=image_type,
                                         metadata=meta, md5=md5,
                                         sha256=sha256)
@@ -696,6 +725,17 @@ class Instance:
     * private_ipv4: str
     * az: str
     * region: str
+    * driver_data: any
+
+    And the following are even more optional (as they are usually
+    already set from the image configuration):
+
+    * username: str
+    * python_path: str
+    * shell_type: str
+    * connection_port: str
+    * connection_type: str
+    * host_keys: [str]
     """
     def __init__(self):
         self.ready = False
@@ -708,6 +748,7 @@ class Instance:
         self.az = None
         self.region = None
         self.metadata = {}
+        self.driver_data = None
 
     def __repr__(self):
         state = []
@@ -781,7 +822,8 @@ class Adapter:
         pass
 
     def getCreateStateMachine(self, hostname, label,
-                              image_external_id, metadata, retries):
+                              image_external_id, metadata, retries,
+                              log):
         """Return a state machine suitable for creating an instance
 
         This method should return a new state machine object
@@ -799,13 +841,15 @@ class Adapter:
             returned from `listInstances`.
         :param retries int: The number of attempts which should be
             made to launch the node.
+        :param log Logger: A logger instance for emitting annotated
+            logs related to the request.
 
         :returns: A :py:class:`StateMachine` object.
 
         """
         raise NotImplementedError()
 
-    def getDeleteStateMachine(self, external_id):
+    def getDeleteStateMachine(self, external_id, log):
         """Return a state machine suitable for deleting an instance
 
         This method should return a new state machine object
@@ -813,6 +857,8 @@ class Adapter:
 
         :param str external_id: The external_id of the instance, as
             supplied by a creation StateMachine or an Instance.
+        :param log Logger: A logger instance for emitting annotated
+            logs related to the request.
         """
         raise NotImplementedError()
 
@@ -881,10 +927,13 @@ class Adapter:
     # The following methods must be implemented only if image
     # management is supported:
 
-    def uploadImage(self, image_name, filename, image_format=None,
-                    metadata=None, md5=None, sha256=None):
+    def uploadImage(self, provider_image, image_name, filename,
+                    image_format=None, metadata=None, md5=None,
+                    sha256=None):
         """Upload the image to the cloud
 
+        :param provider_image ProviderImageConfig:
+            The provider's config for this image
         :param image_name str: The name of the image
         :param filename str: The path to the local file to be uploaded
         :param image_format str: The format of the image (e.g., "qcow")
