@@ -39,19 +39,17 @@ class StaticNodeRequestHandler(NodeRequestHandler):
         return True
 
     def hasRemainingQuota(self, ntype):
-        # A pool of static nodes can manage nodes with different labels.
-        # There is no global quota that we can exceed here. Return true
-        # so we can wait for the required node type and don't block
-        # other node requests.
-        return True
+        # We are always at quota since we cannot launch new nodes.
+        return False
 
     def launch(self, node):
-        self.log.debug("Waiting for node %s to be ready", node.id)
-        self.zk.watchNode(node, self._check_node_state)
+        # NOTE: We do not expect this to be called since hasRemainingQuota()
+        # returning False should prevent the call.
+        raise Exception("Node launching not supported by static driver")
 
     def launchesComplete(self):
-        node_states = [node.state for node in self.nodeset]
-        return all(s in self.DONE_STATES for s in node_states)
+        # We don't wait on a launch since we never actually launch.
+        return True
 
     def checkReusableNode(self, node):
         return self.manager.checkNodeLiveness(node)
