@@ -14,9 +14,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import contextlib
 import logging
 import math
-import operator
 import os
 import os.path
 import socket
@@ -31,8 +31,9 @@ from nodepool import provider_manager
 from nodepool import stats
 from nodepool import config as nodepool_config
 from nodepool import zk
-from nodepool.driver.utils import QuotaInformation
+from nodepool.driver.utils import QuotaInformation, QuotaSupport
 from nodepool.logconfig import get_annotated_logger
+from nodepool.version import version_info as npd_version_info
 
 
 MINS = 60
@@ -109,8 +110,26 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
         # which express a preference for a specific provider.
         launchers = self.zk.getRegisteredLaunchers()
 
-        requests = sorted(self.zk.nodeRequestIterator(),
-                          key=operator.attrgetter("priority"))
+        pm = self.getProviderManager()
+        has_quota_support = isinstance(pm, QuotaSupport)
+        if has_quota_support:
+            # The label quota limits will be used for the whole loop since we
+            # don't want to accept lower priority requests when a label becomes
+            # available after we've already deferred higher priority requests.
+            label_quota = pm.getLabelQuota()
+
+        pool = self.getPoolConfig()
+        pool_labels = set(pool.labels)
+
+        def _sort_key(request):
+            missing_labels = set(request.node_types) - pool_labels
+            # Return a tuple with the number of labels that are *not* served by
+            # this pool and the request priority. This will sort the requests
+            # that we can serve (no missing labels) before the requests that we
+            # need to decline (missing labels > 0).
+            return len(missing_labels), request.priority
+
+        requests = sorted(self.zk.nodeRequestIterator(), key=_sort_key)
         for req in requests:
             if not self.running:
                 return True
@@ -152,8 +171,9 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
             if req.provider and req.provider != self.provider_name:
                 # The request is asking for a specific provider
                 candidate_launchers = set(
-                    [x.id for x in launchers
-                     if x.provider_name == req.provider])
+                    x.id for x in launchers
+                    if x.provider_name == req.provider
+                    and x.supported_labels.issuperset(req.node_types))
                 if candidate_launchers:
                     # There is a launcher online which can satisfy the request
                     if not candidate_launchers.issubset(set(req.declined_by)):
@@ -162,12 +182,18 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
                                   req.provider, candidate_launchers)
                         continue
 
-            pm = self.getProviderManager()
+            if has_quota_support and not all(label_quota.get(l, math.inf) > 0
+                                             for l in req.node_types):
+                # Defer the request as we can't provide the required labels at
+                # the moment.
+                log.debug("Deferring request because labels are unavailable")
+                continue
 
             # check tenant quota if the request has a tenant associated
             # and there are resource limits configured for this tenant
             check_tenant_quota = req.tenant_name and req.tenant_name \
-                in self.nodepool.config.tenant_resource_limits
+                in self.nodepool.config.tenant_resource_limits \
+                and has_quota_support
 
             if check_tenant_quota and not self._hasTenantQuota(req, pm):
                 # Defer request for it to be handled and fulfilled at a later
@@ -192,6 +218,13 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
             # Got a lock, so assign it
             log.info("Assigning node request %s" % req)
 
+            if has_quota_support:
+                # Adjust the label quota so we don't accept more requests
+                # than we have labels available.
+                for label in req.node_types:
+                    with contextlib.suppress(KeyError):
+                        label_quota[label] -= 1
+
             rh = pm.getRequestHandler(self, req)
             rh.run()
             if rh.paused:
@@ -337,6 +370,8 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
 
                 self.updateProviderLimits(
                     self.nodepool.config.providers.get(self.provider_name))
+                self.updateTenantLimits(
+                    self.nodepool.config.tenant_resource_limits)
 
                 if not self.paused_handler:
                     while not self._assignHandlers():
@@ -451,6 +486,14 @@ class CleanupWorker(BaseCleanupWorker):
                         "request", node.id)
                     return
 
+                # If the node is in state init then the launcher that worked
+                # on the lost request has been interrupted between creating
+                # the znode and locking/setting to building. In this case the
+                # znode is leaked and we should delete the node instead of
+                # just deallocating it.
+                if node.state == zk.INIT:
+                    node.state = zk.DELETING
+
                 node.allocated_to = None
                 try:
                     zk_conn.storeNode(node)
@@ -933,7 +976,9 @@ class NodePool(threading.Thread):
             self.zk.connect(configured,
                             tls_cert=config.zookeeper_tls_cert,
                             tls_key=config.zookeeper_tls_key,
-                            tls_ca=config.zookeeper_tls_ca)
+                            tls_ca=config.zookeeper_tls_ca,
+                            timeout=config.zookeeper_timeout,
+                            )
         else:
             self.log.debug("Detected ZooKeeper server changes")
             self.zk.resetHosts(configured)
@@ -956,6 +1001,9 @@ class NodePool(threading.Thread):
         self.reconfigureZooKeeper(config)
         provider_manager.ProviderManager.reconfigure(self.config, config,
                                                      self.getZK())
+        for provider_name in list(config.providers.keys()):
+            if provider_name not in config.provider_managers:
+                del config.providers[provider_name]
         self.setConfig(config)
 
     def removeCompletedRequests(self):
@@ -1090,6 +1138,8 @@ class NodePool(threading.Thread):
         '''
         Start point for the NodePool thread.
         '''
+        self.log.info("Nodepool launcher %s starting",
+                      npd_version_info.release_string())
         while not self._stopped:
             try:
                 self.updateConfig()
