@@ -41,6 +41,10 @@ def get_client():
         return None
 
 
+def normalize_statsd_name(name):
+    return name.replace('.', '_').replace(':', '_')
+
+
 class StatsReporter(object):
     '''
     Class adding statsd reporting functionality.
@@ -72,8 +76,7 @@ class StatsReporter(object):
         if self.handler.request.requestor:
             # Replace '.' which is a graphite hierarchy, and ':' which is
             # a statsd delimeter.
-            requestor = self.handler.request.requestor.replace('.', '_')
-            requestor = requestor.replace(':', '_')
+            requestor = normalize_statsd_name(self.handler.request.requestor)
             keys.append('nodepool.launch.requestor.%s.%s' %
                         (requestor, subkey))
 
@@ -159,3 +162,20 @@ class StatsReporter(object):
         pipeline.gauge(key, max_servers)
 
         pipeline.send()
+
+    def updateTenantLimits(self, tenant_limits):
+        if not self._statsd:
+            return
+
+        pipeline = self._statsd.pipeline()
+        # nodepool.tenant_limits.TENANT.[cores,ram,instances]
+        key_template = 'nodepool.tenant_limits.%s.%s'
+        for tenant, limits in tenant_limits.items():
+            for k, lim in limits.items():
+                # normalize for statsd name, as parts come from arbitrary
+                # user config
+                tenant = normalize_statsd_name(tenant)
+                k = normalize_statsd_name(k)
+                key = key_template % (tenant, k)
+                pipeline.gauge(key, lim)
+        pipeline.send()
