@@ -192,6 +192,12 @@ class BaseTestCase(testtools.TestCase):
         l = logging.getLogger('stevedore')
         l.setLevel(logging.INFO)
         l.propagate = False
+        l = logging.getLogger('botocore')
+        l.setLevel(logging.INFO)
+        l.propagate = False
+        l = logging.getLogger('boto3')
+        l.setLevel(logging.INFO)
+        l.propagate = False
         self.useFixture(fixtures.NestedTempfile())
 
         self.subprocesses = []
@@ -655,6 +661,17 @@ class DBTestCase(BaseTestCase):
         for child in self.zk.client.get_children(node):
             self.printZKTree(join(node, child))
 
+    def getZKTree(self, path, ret=None):
+        """Return the contents of a ZK tree as a dictionary"""
+        if ret is None:
+            ret = {}
+        for key in self.zk.client.get_children(path):
+            subpath = os.path.join(path, key)
+            ret[subpath] = self.zk.client.get(
+                os.path.join(path, key))[0]
+            self.getZKTree(subpath, ret)
+        return ret
+
 
 class IntegrationTestCase(DBTestCase):
     def setUpFakes(self):
