@@ -19,9 +19,7 @@ import fixtures
 import mock
 import time
 
-from pathlib import Path
-
-from nodepool import builder, exceptions, tests
+from nodepool import builder, tests
 from nodepool.driver.fake import provider as fakeprovider
 from nodepool import zk
 from nodepool.config import Config
@@ -29,18 +27,6 @@ from nodepool.nodeutils import iterate_timeout
 
 
 class TestNodepoolBuilderDibImage(tests.BaseTestCase):
-    def test_from_path(self):
-        image = builder.DibImageFile.from_path(
-            '/foo/bar/myid1234.qcow2')
-        self.assertEqual(image.image_id, 'myid1234')
-        self.assertEqual(image.extension, 'qcow2')
-
-    def test_from_path_with_Path(self):
-        image = builder.DibImageFile.from_path(
-            Path('/foo/bar/myid1234.qcow2'))
-        self.assertEqual(image.image_id, 'myid1234')
-        self.assertEqual(image.extension, 'qcow2')
-
     def test_from_image_id(self):
         tempdir = fixtures.TempDir()
         self.useFixture(tempdir)
@@ -69,34 +55,12 @@ class TestNodepoolBuilderDibImage(tests.BaseTestCase):
         self.assertEqual(images[0].extension, 'qcow2')
         self.assertEqual(images[1].extension, 'raw')
 
-    def test_from_images_dir(self):
-        tempdir = fixtures.TempDir()
-        self.useFixture(tempdir)
-        image_path_1 = os.path.join(tempdir.path, 'myid1234.qcow2')
-        image_path_2 = os.path.join(tempdir.path, 'myid1234.raw')
-        open(image_path_1, 'w')
-        open(image_path_2, 'w')
-
-        images = builder.DibImageFile.from_images_dir(tempdir.path)
-        images = sorted(images, key=lambda x: x.extension)
-        self.assertEqual(len(images), 2)
-
-        self.assertEqual(images[0].image_id, 'myid1234')
-        self.assertEqual(images[0].extension, 'qcow2')
-        self.assertEqual(images[1].image_id, 'myid1234')
-        self.assertEqual(images[1].extension, 'raw')
-
     def test_to_path(self):
         image = builder.DibImageFile('myid1234', 'qcow2')
         self.assertEqual(image.to_path('/imagedir'),
                          '/imagedir/myid1234.qcow2')
         self.assertEqual(image.to_path('/imagedir/'),
                          '/imagedir/myid1234.qcow2')
-        self.assertEqual(image.to_path('/imagedir/', False),
-                         '/imagedir/myid1234')
-
-        image = builder.DibImageFile('myid1234')
-        self.assertRaises(exceptions.BuilderError, image.to_path, '/imagedir/')
 
 
 class TestNodepoolBuilderImageInheritance(tests.BaseTestCase):
