@@ -476,6 +476,7 @@ class NodeRequest(BaseModel):
         self.provider = None
         self.relative_priority = 0
         self.event_id = None
+        self.created_time = None
         self.tenant_name = None
 
     def __repr__(self):
@@ -495,6 +496,7 @@ class NodeRequest(BaseModel):
                     self.requestor_data == other.requestor_data and
                     self.provider == other.provider and
                     self.relative_priority == other.relative_priority and
+                    self.created_time == other.created_time and
                     self.tenant_name == other.tenant_name)
         else:
             return False
@@ -521,6 +523,7 @@ class NodeRequest(BaseModel):
         d['provider'] = self.provider
         d['relative_priority'] = self.relative_priority
         d['event_id'] = self.event_id
+        d['created_time'] = self.created_time
         d['tenant_name'] = self.tenant_name
         return d
 
@@ -550,6 +553,7 @@ class NodeRequest(BaseModel):
         self.provider = d.get('provider')
         self.relative_priority = d.get('relative_priority', 0)
         self.event_id = d.get('event_id')
+        self.created_time = d.get('created_time')
         self.tenant_name = d.get('tenant_name')
 
 
@@ -594,6 +598,8 @@ class Node(BaseModel):
         self.attributes = None
         self.python_path = None
         self.tenant_name = None
+        self.driver_data = None
+        self.requestor = None
 
     def __repr__(self):
         d = self.toDict()
@@ -635,7 +641,9 @@ class Node(BaseModel):
                     self.resources == other.resources and
                     self.attributes == other.attributes and
                     self.python_path == other.python_path and
-                    self.tenant_name == other.tenant_name)
+                    self.tenant_name == other.tenant_name and
+                    self.driver_data == other.driver_data and
+                    self.requestor == other.requestor)
         else:
             return False
 
@@ -688,6 +696,8 @@ class Node(BaseModel):
         d['attributes'] = self.attributes
         d['python_path'] = self.python_path
         d['tenant_name'] = self.tenant_name
+        d['driver_data'] = self.driver_data
+        d['requestor'] = self.requestor
         return d
 
     @staticmethod
@@ -755,6 +765,8 @@ class Node(BaseModel):
         self.python_path = d.get('python_path')
         self.shell_type = d.get('shell_type')
         self.tenant_name = d.get('tenant_name')
+        self.driver_data = d.get('driver_data')
+        self.requestor = d.get('requestor')
 
 
 class ZooKeeper(object):
@@ -1000,7 +1012,7 @@ class ZooKeeper(object):
         self._became_lost = False
 
     def connect(self, host_list, read_only=False, tls_cert=None,
-                tls_key=None, tls_ca=None):
+                tls_key=None, tls_ca=None, timeout=10.0):
         '''
         Establish a connection with ZooKeeper cluster.
 
@@ -1018,8 +1030,11 @@ class ZooKeeper(object):
         '''
         if self.client is None:
             hosts = buildZooKeeperHosts(host_list)
-            args = dict(hosts=hosts,
-                        read_only=read_only)
+            args = dict(
+                hosts=hosts,
+                read_only=read_only,
+                timeout=timeout,
+            )
 
             args['use_ssl'] = True
             if not (tls_key and tls_cert and tls_ca):
@@ -1943,7 +1958,8 @@ class ZooKeeper(object):
         request.lock.release()
         request.lock = None
 
-    def lockNode(self, node, blocking=True, timeout=None):
+    def lockNode(self, node, blocking=True, timeout=None,
+                 ephemeral=True, identifier=None):
         '''
         Lock a node.
 
@@ -1957,6 +1973,10 @@ class ZooKeeper(object):
             acquire the lock
         :param int timeout: When blocking, how long to wait for the lock
             to get acquired. None, the default, waits forever.
+        :param bool ephemeral: Whether to use an ephemeral lock.  Unless
+            you have a really good reason, use the default of True.
+        :param bool identifier: Identifies the lock holder.  The default
+            of None is usually fine.
 
         :raises: TimeoutException if we failed to acquire the lock when
             blocking with a timeout. ZKLockException if we are not blocking
@@ -1964,8 +1984,8 @@ class ZooKeeper(object):
         '''
         path = self._nodeLockPath(node.id)
         try:
-            lock = Lock(self.client, path)
-            have_lock = lock.acquire(blocking, timeout)
+            lock = Lock(self.client, path, identifier)
+            have_lock = lock.acquire(blocking, timeout, ephemeral)
         except kze.LockTimeout:
             raise npe.TimeoutException(
                 "Timeout trying to acquire lock %s" % path)
@@ -1998,6 +2018,26 @@ class ZooKeeper(object):
         node.lock.release()
         node.lock = None
 
+    def forceUnlockNode(self, node):
+        '''
+        Forcibly unlock a node.
+
+        :param Node node: The node to unlock.
+        '''
+        path = self._nodeLockPath(node.id)
+        try:
+            self.client.delete(path, recursive=True)
+        except kze.NoNodeError:
+            pass
+
+    def getNodeLockContenders(self, node):
+        '''
+        Return the contenders for a node lock.
+        '''
+        path = self._nodeLockPath(node.id)
+        lock = Lock(self.client, path)
+        return lock.contenders()
+
     def getNodes(self):
         '''
         Get the current list of all nodes.
@@ -2242,7 +2282,12 @@ class ZooKeeper(object):
                                 cache.
         '''
 
-        node_ids = self._cached_nodes.keys() if cached_ids else self.getNodes()
+        if cached_ids:
+            # get a copy of the keys view to avoid runtime errors in the event
+            # the _cached_nodes dict gets updated while iterating
+            node_ids = list(self._cached_nodes.keys())
+        else:
+            node_ids = self.getNodes()
 
         for node_id in node_ids:
             node = self.getNode(node_id, cached=cached)
@@ -2496,3 +2541,46 @@ class ZooKeeper(object):
     def getStatsElection(self, identifier):
         path = self._electionPath('stats')
         return Election(self.client, path, identifier)
+
+    def exportImageData(self):
+        '''
+        Export the DIB image and upload data from ZK for backup purposes.
+        '''
+        ret = {}
+        for image_name in self.getImageNames():
+            paused = self.getImagePaused(image_name)
+            if paused:
+                paused_path = self._imagePausePath(image_name)
+                ret[paused_path] = ''
+            for build_no in self.getBuildNumbers(image_name):
+                build_path = self._imageBuildsPath(image_name) + "/" + build_no
+                try:
+                    build_data, stat = self.client.get(build_path)
+                except kze.NoNodeError:
+                    continue
+                ret[build_path] = build_data.decode('utf8')
+                for provider_name in self.getBuildProviders(image_name,
+                                                            build_no):
+                    for upload_no in self.getImageUploadNumbers(
+                            image_name, build_no, provider_name):
+                        upload_path = self._imageUploadPath(
+                            image_name, build_no, provider_name) + "/"
+                        upload_path += upload_no
+                        try:
+                            upload_data, stat = self.client.get(upload_path)
+                        except kze.NoNodeError:
+                            continue
+                        ret[upload_path] = upload_data.decode('utf8')
+        return ret
+
+    def importImageData(self, import_data):
+        '''Import the DIB image and upload data to ZK.
+
+        This makes no guarantees about locking; it is expected to be
+        run on a quiescent system with no daemons running.
+
+        '''
+        for path, data in import_data.items():
+            self.client.create(path,
+                               value=data.encode('utf8'),
+                               makepath=True)
